package de.ozgcloud.bescheid;

import java.util.Objects;

import jakarta.annotation.PostConstruct;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import de.ozgcloud.bescheid.vorgang.VorgangService;
import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.extern.log4j.Log4j2;

@Service
@Log4j2
class BescheidService {

	// TODO: Fehlernachricht anpassen
	private static final String ERROR_MESSAGE_NO_SERVICE = "'CREATE_BESCHEID' Command received but no Bescheid Endpoint is configured.";

	@Autowired
	private VorgangService vorgangService;
	@Autowired(required = false)
	private BescheidRemoteService remoteService;

	@PostConstruct
	void logStatus() {
		if (Objects.isNull(remoteService)) {
			LOG.info("No BescheidRemoteService configured - Bescheid creation is not possible.");
		} else {
			LOG.info("Bescheid-Manager is configured.");
		}
	}

	public Bescheid createBescheid(BescheidRequest request) {
		checkRemoteService();

		return doCreateBescheid(request);
	}

	private Bescheid doCreateBescheid(BescheidRequest request) {
		var vorgang = vorgangService.getById(request.getVorgangId());
		return remoteService.create(request, vorgang)
				.toBuilder().vorgangId(vorgang.getId()).serviceKonto(vorgang.getServiceKonto())
				.build();
	}

	private void checkRemoteService() {
		if (Objects.isNull(remoteService)) {
			LOG.error(ERROR_MESSAGE_NO_SERVICE);
			throw new TechnicalException(ERROR_MESSAGE_NO_SERVICE);
		}
	}

}
