/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.bescheid.attacheditem;

import static java.util.Objects.*;

import java.util.Collection;
import java.util.List;
import java.util.Optional;
import java.util.stream.Stream;

import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import de.ozgcloud.bescheid.GrpcBescheid;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemResponse;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItem;
import lombok.RequiredArgsConstructor;

@Component
@RequiredArgsConstructor
@Deprecated(since = "2.6.0")
public class BescheidItemMapper {

	private final GrpcObjectMapper grpcObjectMapper;

	public Stream<BescheidItem> fromFindVorgangAttachedItemResponse(GrpcFindVorgangAttachedItemResponse grpcResponse) {
		return grpcResponse.getVorgangAttachedItemsList().stream()
				.map(this::mapFromVorgangAttachedItem);
	}

	BescheidItem mapFromVorgangAttachedItem(GrpcVorgangAttachedItem item) {
		return BescheidItem.builder()
				.id(item.getId())
				.version(item.getVersion())
				.client(item.getClient())
				.vorgangId(item.getVorgangId())
				.itemName(item.getItemName())
				.bescheidData(grpcObjectMapper.mapFromGrpc(item.getItem()))
				.build();
	}

	public GrpcBescheid toBescheid(BescheidItem bescheid) {
		var bescheidBuilder = GrpcBescheid.newBuilder().setVersion(bescheid.getVersion());
		Optional.ofNullable(bescheid.getId()).ifPresent(bescheidBuilder::setId);
		var bescheidData = bescheid.getBescheidData();
		Optional.ofNullable(MapUtils.getString(bescheidData, BescheidItem.FIELD_BESCHIEDEN_AM)).map(StringUtils::trimToNull)
				.ifPresent(bescheidBuilder::setBeschiedenAm);
		Optional.ofNullable(MapUtils.getBoolean(bescheidData, BescheidItem.FIELD_BEWILLIGT)).ifPresent(bescheidBuilder::setBewilligt);
		Optional.ofNullable(MapUtils.getString(bescheidData, BescheidItem.FIELD_BESCHEID_DOCUMENT)).map(StringUtils::trimToNull)
				.ifPresent(bescheidBuilder::setBescheidDocument);
		Optional.ofNullable(MapUtils.getString(bescheidData, BescheidItem.FIELD_SEND_BY)).map(StringUtils::trimToNull)
				.ifPresent(bescheidBuilder::setSendBy);
		Optional.ofNullable(MapUtils.getString(bescheidData, BescheidItem.FIELD_NACHRICHT_TEXT)).map(StringUtils::trimToNull)
				.ifPresent(bescheidBuilder::setNachrichtText);
		Optional.ofNullable(MapUtils.getString(bescheidData, BescheidItem.FIELD_NACHRICHT_SUBJECT)).map(StringUtils::trimToNull)
				.ifPresent(bescheidBuilder::setNachrichtSubject);
		Optional.ofNullable(bescheidData.get(BescheidItem.FIELD_ATTACHMENTS)).map(this::toAttachments).ifPresent(bescheidBuilder::addAllAttachments);
		return bescheidBuilder.build();
	}

	@SuppressWarnings("unchecked")
	Collection<String> toAttachments(Object attachmentsObject) {
		if (attachmentsObject instanceof Collection<?> attachments) {
			return (Collection<String>) attachments;
		}
		if (attachmentsObject instanceof String attachment) {
			attachment = StringUtils.trimToNull(attachment);
			if (nonNull(attachment)) {
				return List.of(attachment);
			}
		} else if (nonNull(attachmentsObject)) {
			return List.of(String.valueOf(attachmentsObject));
		}
		return null; //NOSONAR
	}
}
