package de.ozgcloud.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.bescheid.attacheditem.AttachedItemService;
import de.ozgcloud.bescheid.attacheditem.BescheidItemMapper;
import de.ozgcloud.bescheid.attacheditem.BescheidItemTestFactory;
import io.grpc.stub.StreamObserver;

class BescheidGrpcServiceTest {

	@Spy
	@InjectMocks
	private BescheidGrpcService service;

	@Mock
	private AttachedItemService attachedItemService;
	@Mock
	private BescheidItemMapper mapper;

	@Nested
	class TestGetBescheidDraft {

		@Mock
		private GrpcGetBescheidDraftRequest request;
		@Mock
		private StreamObserver<GrpcGetBescheidDraftResponse> responseObserver;
		@Captor
		private ArgumentCaptor<GrpcGetBescheidDraftResponse> responseCaptor;

		private static final GrpcGetBescheidDraftResponse response = GrpcGetBescheidDraftResponse.newBuilder().build();

		@Test
		void shouldBuildResponse() {
			when(mapper.toBescheid(any())).thenReturn(GrpcBescheidTestFactory.create());
			when(attachedItemService.findBescheidItem(any())).thenReturn(Optional.of(BescheidItemTestFactory.create()));

			service.getBescheidDraft(request, responseObserver);

			verify(service).buildResponse(any());
		}

		@Test
		void shouldCallNext() {
			when(mapper.toBescheid(any())).thenReturn(GrpcBescheidTestFactory.create());
			when(service.buildResponse(any())).thenReturn(response);
			when(attachedItemService.findBescheidItem(any())).thenReturn(Optional.of(BescheidItemTestFactory.create()));

			service.getBescheidDraft(request, responseObserver);

			verify(responseObserver).onNext(response);
		}

		@Test
		void shouldCallNextWhenNoBescheid() {
			service.getBescheidDraft(request, responseObserver);

			verify(responseObserver).onNext(responseCaptor.capture());
			assertThat(responseCaptor.getValue().hasBescheid()).isFalse();
		}

		@Test
		void shouldCallCompleted() {
			service.getBescheidDraft(request, responseObserver);

			verify(responseObserver).onCompleted();
		}

	}

	@Nested
	class TestBuildResponse {

		@Test
		void shouldCallMapper() {
			when(mapper.toBescheid(any())).thenReturn(GrpcBescheidTestFactory.create());
			var bescheid = BescheidItemTestFactory.create();

			service.buildResponse(bescheid);

			verify(mapper).toBescheid(bescheid);
		}

		@Test
		void shouldSetBescheid() {
			var grpcBescheid = GrpcBescheidTestFactory.create();
			when(mapper.toBescheid(any())).thenReturn(grpcBescheid);

			var response = service.buildResponse(BescheidItemTestFactory.create());

			assertThat(response.getBescheid()).isEqualTo(grpcBescheid);
		}
	}

}
