package de.ozgcloud.bescheid;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.bescheid.vorgang.Vorgang;
import de.ozgcloud.bescheid.vorgang.VorgangService;
import de.ozgcloud.bescheid.vorgang.VorgangTestFactory;

class BescheidServiceTest {

	@InjectMocks
	private BescheidService service;

	@Mock
	private VorgangService vorgangService;
	@Mock
	private BescheidRemoteService remoteService;

	@Nested
	class TestCreateBescheid {

		private Vorgang vorgang = VorgangTestFactory.create();
		private Bescheid bescheid = BescheidTestFactory.createBuilder().vorgangId(null).build();

		@BeforeEach
		void initMocks() {
			when(vorgangService.getById(any())).thenReturn(vorgang);
			when(remoteService.create(any(), any())).thenReturn(bescheid);
		}

		@Test
		void shouldLoadVorgang() {
			service.createBescheid(BescheidRequestTestFactory.create());

			verify(vorgangService).getById(VorgangTestFactory.ID);
		}

		@Test
		void shouldCreateBescheid() {
			BescheidRequest request = BescheidRequestTestFactory.create();

			service.createBescheid(request);

			verify(remoteService).create(request, vorgang);
		}

		@Test
		void shouldReturnCreatedBescheid() {
			var result = service.createBescheid(BescheidRequestTestFactory.create());

			assertThat(result).usingRecursiveComparison().isEqualTo(BescheidTestFactory.create());
		}
	}

}
