/*
 * Copyright (C) 2024 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.bescheid.attacheditem;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import de.ozgcloud.bescheid.attacheditem.AttachedItem.AttachedItemBuilder;
import de.ozgcloud.command.CommandTestFactory;
import de.ozgcloud.document.Document;
import de.ozgcloud.document.DocumentService;
import de.ozgcloud.document.DocumentTestFactory;

public class AttachedItemTestFactory {

	public static final String ID = "bescheid-item-id";
	public static final long VERSION = 10L;
	public static final String CLIENT = "client";

	public static final String BESCHEIDEN_AM = "2024-01-01";
	public static final String BESCHEID_DOCUMENT = "bescheid-document";
	public static final String ATTACHMENT = "attachment-id";
	public static final String SEND_BY = "POSTFACH";
	public static final String NACHRICHT_TEXT = "nachricht-text";
	public static final String NACHRICHT_SUBJECT = "nachricht-subject";

	public static AttachedItem createBescheid() {
		return createBuilder()
				.itemName(AttachedItemService.BESCHEID_ITEM_NAME)
				.item(createBescheidItem())
				.build();
	}

	public static Map<String, Object> createBescheidItem() {
		return Map.of(
				BescheidItem.FIELD_STATUS, BescheidItem.Status.DRAFT,
				BescheidItem.FIELD_BESCHIEDEN_AM, BESCHEIDEN_AM,
				BescheidItem.FIELD_BEWILLIGT, true,
				BescheidItem.FIELD_BESCHEID_DOCUMENT, BESCHEID_DOCUMENT,
				BescheidItem.FIELD_ATTACHMENTS, List.of(ATTACHMENT),
				BescheidItem.FIELD_SEND_BY, SEND_BY,
				BescheidItem.FIELD_NACHRICHT_TEXT, NACHRICHT_TEXT,
				BescheidItem.FIELD_NACHRICHT_SUBJECT, NACHRICHT_SUBJECT);
	}

	public static AttachedItem createDocument() {
		return createDocumentBuilder().build();
	}

	public static AttachedItem.AttachedItemBuilder createDocumentBuilder() {
		return createBuilder()
				.itemName(DocumentService.DOCUMENT_ITEM_NAME)
				.item(createDocumentItem());
	}

	public static HashMap<String, Object> createDocumentItem() {
		var map = new HashMap<String, Object>();
		map.put(Document.FIELD_DOCUMENT_TYPE, DocumentService.DOCUMENT_TYPE);
		map.put(Document.FIELD_DOCUMENT_FILE, DocumentTestFactory.DOCUMENT_FILE);
		map.put(Document.FIELD_NACHRICHT_TEXT, DocumentTestFactory.NACHRICHT_TEXT);
		map.put(Document.FIELD_NACHRICHT_SUBJECT, DocumentTestFactory.NACHRICHT_SUBJECT);
		return map;
	}

	public static AttachedItemBuilder createBuilder() {
		return AttachedItem.builder()
				.id(ID)
				.version(VERSION)
				.client(CLIENT)
				.vorgangId(CommandTestFactory.VORGANG_ID);
	}

}
