package de.ozgcloud.bescheid.nachricht;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;

class NachrichtMapperTest {

	@InjectMocks
	private NachrichtMapper mapper = Mappers.getMapper(NachrichtMapper.class);;

	@Nested
	class TestMapToGrpc {
		@Test
		void shouldMapNachricht() {
			var mapped = mapper.mapToGrpc(NachrichtTestFactory.create());

			assertThat(mapped).usingRecursiveComparison()
					.ignoringFields("replyOption_", "memoizedHashCode")
					.isEqualTo(GrpcPostfachNachrichtTestFactory.create());
		}

		@Test
		void shouldIgnoreMissingFileId() {
			var mapped = mapper.mapToGrpc(NachrichtTestFactory.createBuilder().bescheidFileId(null).build());

			assertThat(mapped.getAttachmentList()).isEmpty();
		}

		@Test
		void shouldSetReplyOption() {
			var mapped = mapper.mapToGrpc(NachrichtTestFactory.create());

			assertThat(mapped.getReplyOption()).isEqualTo("FORBIDDEN");
		}

		@Test
		void shouldNotMapId() {
			var mapped = mapper.mapToGrpc(NachrichtTestFactory.create());

			assertThat(mapped.getId()).isEmpty();
		}
	}

}
