package de.ozgcloud.bescheid.nachricht;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.util.Optional;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.bescheid.BescheidTestFactory;
import de.ozgcloud.bescheid.vorgang.VorgangTestFactory;

class NachrichtServiceTest {

	@Spy
	@InjectMocks
	private NachrichtService service;

	@Mock
	private NachrichtRemoteService remoteService;

	@Nested
	class TestCreateNachrichtDraft {

		private Nachricht nachricht = NachrichtTestFactory.create();

		@Test
		void shouldCallRemoteService() {
			doReturn(Optional.of(nachricht)).when(service).buildNachricht(any());

			service.createNachrichtDraft(BescheidTestFactory.create());

			verify(remoteService).saveDraft(nachricht);
		}

		@Nested
		class BuildNachricht {

			@BeforeEach
			void init() {
				doReturn(NachrichtTestFactory.MAIL_BODY).when(service).buildMessage(any());
			}

			@Test
			void shouldFillMailBody() {
				var nachricht = service.buildNachricht(BescheidTestFactory.create()).get();

				assertThat(nachricht.getMailBody()).isEqualTo(NachrichtTestFactory.MAIL_BODY);
			}

			@Test
			void shouldSetSubject() {
				var nachricht = service.buildNachricht(BescheidTestFactory.create()).get();

				assertThat(nachricht.getSubject()).isEqualTo(NachrichtService.SUBJECT);
			}

			@Test
			void shouldSetUser() {
				var nachricht = service.buildNachricht(BescheidTestFactory.create()).get();

				assertThat(nachricht.getCreatedBy()).isEqualTo(BescheidTestFactory.CREATED_BY);
			}

			@Test
			void shouldSetVorgangId() {
				var nachricht = service.buildNachricht(BescheidTestFactory.create()).get();

				assertThat(nachricht.getVorgangId()).isEqualTo(VorgangTestFactory.ID);
			}
		}
	}

	@Nested
	class TestBuildMessage {
		@Test
		void shouldUseTextFromBescheid() {
			var message = service.buildMessage(BescheidTestFactory.create());

			assertThat(message).isEqualTo(BescheidTestFactory.NACHRICHT_TEXT);
		}

		@Test
		void shouldUseDefaultTemplate() {
			doReturn("FROM_TEMPLATE").when(service).fillTemplate(any(), any());

			var message = service.buildMessage(BescheidTestFactory.createBuilder().nachrichtText(Optional.empty()).build());

			assertThat(message).isEqualTo("FROM_TEMPLATE");
		}
	}
}
