package de.ozgcloud.bescheid.smartdocuments;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.File;
import java.util.Optional;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;

import de.ozgcloud.bescheid.BescheidRequestTestFactory;
import de.ozgcloud.bescheid.BescheidTestFactory;
import de.ozgcloud.bescheid.common.callcontext.UserProfileTestFactory;
import de.ozgcloud.bescheid.smartdocuments.SmartDocumentsRequest.CustomerData.UserData;
import de.ozgcloud.bescheid.vorgang.VorgangTestFactory;
import de.ozgcloud.common.binaryfile.TempFileUtils;
import de.ozgcloud.common.test.TestUtils;

class SmartDocumentsBescheidRemoteServiceTest {

	@Spy
	@InjectMocks
	private SmartDocumentsBescheidRemoteService service;

	@Mock
	private SmartDocumentsProperties properties;

	@Nested
	class TestBuildBescheid {

		@Test
		void shouldFillBescheid() {
			var bescheid = service.buildBescheid(BescheidRequestTestFactory.create(), SmartDocumentsResponseTestFactory.create());

			assertThat(bescheid).usingRecursiveComparison().ignoringFields("serviceKonto")
					.isEqualTo(BescheidTestFactory.createBuilder().nachrichtText(Optional.empty()).build());
		}
	}

	@Nested
	class TestCreateRequest {

		@Test
		void shouldFillUserData() {
			var request = service.createRequest(BescheidRequestTestFactory.create(), VorgangTestFactory.create());

			assertThat(request.getCustomerData().getUserData()).isNotNull()
					.extracting(UserData::getEmail).isEqualTo(UserProfileTestFactory.EMAIL);
		}
	}

	@Nested
	class TestGetNachrichtText {
		private File xmlFile = TempFileUtils.writeTmpFile(TestUtils.loadFile("SD_answer.xml"));

		@Test
		void shouldCallExtractText() {
			doReturn(Optional.of(xmlFile)).when(service).getXMLFile(any());

			service.getNachrichtText(SmartDocumentsResponseTestFactory.create());

			verify(service).extractTextFormXmlFile(notNull());
		}

		@Nested
		class ExtractingText {

			@Test
			void shouldReturnText() {
				var text = service.extractTextFormXmlFile(xmlFile);

				assertThat(text).isPresent().get().isNotNull();
			}

			@Test
			void shouldHandleError() {
				var text = service.extractTextFormXmlFile(null);

				assertThat(text).isEmpty();
			}

		}
	}
}
