/*
 * Copyright (c) 2024. Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.nachrichten.antragraum;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.util.Objects;

import org.opensaml.core.xml.XMLObject;
import org.opensaml.core.xml.config.XMLObjectProviderRegistrySupport;
import org.opensaml.core.xml.io.UnmarshallingException;
import org.opensaml.saml.saml2.core.Response;
import org.opensaml.saml.saml2.core.impl.ResponseUnmarshaller;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.security.saml2.Saml2Exception;
import org.springframework.stereotype.Service;

import lombok.RequiredArgsConstructor;
import net.shibboleth.utilities.java.support.xml.XMLParserException;

@Service
@ConditionalOnProperty(AntragraumProperties.PROPERTY_ANTRAGSRAUM_URL)
@RequiredArgsConstructor
class Saml2Parser {
	private final BayernIdSamlConfiguration configuration;
	private ResponseUnmarshaller unmarshaller;

	Response parse(String request) {
		return (Response) xmlObject(new ByteArrayInputStream(request.getBytes(StandardCharsets.UTF_8)));
	}

	XMLObject xmlObject(InputStream inputStream) throws Saml2Exception {
		try {
			var document = configuration.getParserPool().parse(inputStream);
			var element = document.getDocumentElement();
			return getUnmarshaller().unmarshall(element);
		} catch (XMLParserException | UnmarshallingException e) {
			throw new Saml2Exception("Failed to deserialize LogoutRequest", e);
		}
	}

	private ResponseUnmarshaller getUnmarshaller() {
		if (Objects.nonNull(unmarshaller)) {
			return unmarshaller;
		}

		unmarshaller = (ResponseUnmarshaller) XMLObjectProviderRegistrySupport.getUnmarshallerFactory()
				.getUnmarshaller(Response.DEFAULT_ELEMENT_NAME);
		return unmarshaller;
	}
}
