package de.ozgcloud.nachrichten.info;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;

import de.ozgcloud.nachrichten.NachrichtenManagerProperties;
import de.ozgcloud.nachrichten.antragraum.AntragraumProperties;
import de.ozgcloud.nachrichten.postfach.PostfachException;
import de.ozgcloud.nachrichten.postfach.PostfachMessageCode;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@ConditionalOnProperty(AntragraumProperties.PROPERTY_ANTRAGSRAUM_URL)
public class InfoManagerService {

	private final InfoManagerRemoteService infomanagerRemoteService;
	private final NachrichtenManagerProperties nachrichtenManagerProperties;

	public void notifyInfoManager(PostfachNachricht postfachNachricht) {
		try {
			infomanagerRemoteService.sendNotification(builInfomanagerRequest(postfachNachricht));
		} catch (RuntimeException e) {
			throw new PostfachException("Error while sending notification to Antragsraum", PostfachMessageCode.PROCESS_FAILED_MESSAGE_CODE, e);
		}
	}

	InfoManagerRequest builInfomanagerRequest(PostfachNachricht postfachNachricht) {
		return InfoManagerRequest.builder()
				.nachrichtId(postfachNachricht.getId())
				.vorgangId(postfachNachricht.getVorgangId())
				.postfachId(postfachNachricht.getPostfachId())
				.nachrichtenManagerUrl(nachrichtenManagerProperties.getUrl())
				.build();
	}
}
