/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.nachrichten.antragraum;

import static org.assertj.core.api.Assertions.*;

import java.time.format.DateTimeFormatter;

import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import de.ozgcloud.nachrichten.postfach.PostfachNachrichtTestFactory;
import de.ozgcloud.nachrichten.postfach.osi.MessageTestFactory;

class AntragraumNachrichtMapperTest {

	AntragraumNachrichtMapper mapper = Mappers.getMapper(AntragraumNachrichtMapper.class);

	@Test
	void shouldMapVorgangId() {
		var result = map();

		assertThat(result.getVorgangId()).isEqualTo(MessageTestFactory.VORGANG_ID);
	}

	@Test
	void shouldMapId() {
		var result = map();

		assertThat(result.getId()).isEqualTo(PostfachNachrichtTestFactory.ID);
	}

	@Test
	void shouldMapVorgangName() {
		var result = map();

		assertThat(result.getVorgangName()).isEqualTo(MessageTestFactory.SUBJECT);
	}

	@Test
	void shouldMapStatus() {
		var result = map();

		assertThat(result.getStatus()).isEqualTo(AntragraumNachrichtMapper.DEFAULT_STATUS);
	}

	@Test
	void shouldMapSentAt() {
		var result = map();

		assertThat(result.getSentAt()).isEqualTo(PostfachNachrichtTestFactory.SENT_AT.format(DateTimeFormatter.ISO_LOCAL_DATE_TIME));
	}

	@Test
	void shouldMapText() {
		var result = map();

		assertThat(result.getText()).isEqualTo(PostfachNachrichtTestFactory.MAIL_BODY);
	}

	@Test
	void shouldMapAttachments() {
		var result = map();

		assertThat(result.getAttachmentFileIdCount()).isEqualTo(1);
		assertThat(result.getAttachmentFileId(0)).isEqualTo(PostfachNachrichtTestFactory.ATTACHMENTS.get(0));
	}

	private GrpcRueckfrage map() {
		return mapper.toGrpc(PostfachNachrichtTestFactory.create());
	}
}