/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.nachrichten.antragraum;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.springframework.core.io.InputStreamResource;

import de.ozgcloud.common.test.TestUtils;

class BayernIdSamlConfigurationTest {
	@Mock
	private AntragraumProperties properties;
	@InjectMocks
	private BayernIdSamlConfiguration bayernIdSamlConfiguration;

	@Nested
	class TestCreationOfParserPool {
		@BeforeEach
		void setup() throws Exception {
			bayernIdSamlConfiguration.initOpenSAML();
		}

		@Test
		void shouldCreateParserPool() {
			assertThat(bayernIdSamlConfiguration.getParserPool()).isNotNull();
		}
	}

	@Nested
	class TestCreatingTrustEngine {
		@BeforeEach
		void setup() throws Exception {
			when(properties.getMetadataUri()).thenReturn(new InputStreamResource(TestUtils.loadFile("bayernid-idp-infra.xml")));
			when(properties.getEntityId()).thenReturn("https://antragsraum.ozgcloud.de/");
			bayernIdSamlConfiguration.initOpenSAML();
		}

		@Test
		void shouldCreateTrustEngine() {
			assertThat(bayernIdSamlConfiguration.getTrustEngine()).isNotNull();
		}
	}

	@Nested
	class TestLoadingVerficationData {
		@Test
		void shouldCreateVerificationCriteria() {
			when(properties.getEntityId()).thenReturn("https://antragsraum.ozgcloud.de/");
			bayernIdSamlConfiguration.initOpenSAML();

			assertThat(bayernIdSamlConfiguration.getVerificationCriteria()).isNotNull();
		}

		@Test
		void shouldLoadVerificationCert() throws Exception {
			when(properties.getMetadataUri()).thenReturn(new InputStreamResource(TestUtils.loadFile("bayernid-idp-infra.xml")));
			bayernIdSamlConfiguration.initOpenSAML();

			assertThat(bayernIdSamlConfiguration.getCertificatesFromMetadata()).isNotNull();
		}
	}
}