/*
 * Copyright (c) 2024.
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

package de.ozgcloud.nachrichten.antragraum;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Disabled;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.opensaml.saml.saml2.core.Attribute;
import org.opensaml.saml.saml2.core.AttributeStatement;
import org.opensaml.saml.saml2.core.Response;
import org.springframework.core.io.InputStreamResource;
import org.springframework.test.util.ReflectionTestUtils;

import de.ozgcloud.common.test.TestUtils;

@Disabled("Passende certificat und key file muessen local vorhanden sein")
class Saml2DecrypterTest {
	private Response samlResponse;

	@Mock
	private AntragraumProperties properties;

	private BayernIdSamlConfiguration configuration;

	private Saml2Decrypter decrypter;

	@BeforeEach
	void setup() throws Exception {
		when(properties.getDecryptionCertificate()).thenReturn(new InputStreamResource(TestUtils.loadFile("bayernid-test-enc.crt")));
		when(properties.getDecryptionPrivateKey()).thenReturn(new InputStreamResource(TestUtils.loadFile("bayernid-test-enc.key")));

		configuration = new BayernIdSamlConfiguration();
		ReflectionTestUtils.setField(configuration, "antragraumProperties", properties);
		configuration.initOpenSAML();

		var samlResponseString = TestUtils.loadTextFile("SamlResponse.xml");
		var parser = new Saml2Parser(configuration);
		samlResponse = parser.parse(samlResponseString);

		decrypter = new Saml2Decrypter(configuration);
		decrypter.init();
	}

	@Test
	void shouldDecrypt() {
		assertThat(samlResponse.getAssertions()).isEmpty();

		decrypter.decryptResponseElements(samlResponse);

		assertThat(samlResponse.getAssertions()).isNotEmpty();
	}

	@Test
	void shouldHaveSubject() {
		decrypter.decryptResponseElements(samlResponse);
		var samlAssertion = samlResponse.getAssertions().get(0);

		assertThat(samlAssertion.getSubject()).isNotNull();
	}

	@Test
	void shouldHaveAuthnStatements() {
		decrypter.decryptResponseElements(samlResponse);
		var samlAssertion = samlResponse.getAssertions().get(0);
		var authnStatements = samlAssertion.getAuthnStatements();

		assertThat(authnStatements).isNotNull();
	}

	@Test
	void shouldHaveStatements() {
		decrypter.decryptResponseElements(samlResponse);
		var samlAssertion = samlResponse.getAssertions().get(0);
		var statements = samlAssertion.getStatements();

		assertThat(statements).isNotNull();
	}

	@Test
	void shouldHaveAttributes() {
		decrypter.decryptResponseElements(samlResponse);
		var samlAssertion = samlResponse.getAssertions().get(0);
		var statements = (AttributeStatement) samlAssertion.getStatements().get(1);
		var attributes = statements.getAttributes();
		assertThat(attributes).hasSize(7);
	}

	@Test
	void shouldHavePostfachId() {
		decrypter.decryptResponseElements(samlResponse);
		var samlAssertion = samlResponse.getAssertions().get(0);
		var statements = (AttributeStatement) samlAssertion.getStatements().get(1);
		var attributes = statements.getAttributes();
		var postfachIdOptional = attributes.stream().filter(attribute -> "legacyPostkorbHandle".equals(attribute.getFriendlyName())).findFirst();
		assertThat(postfachIdOptional).map(Attribute::getAttributeValues).isNotNull();
	}

	@Test
	void shouldGetPostfachId() {
		var postfachId = decrypter.decryptPostfachId(samlResponse);

		assertThat(postfachId).isEqualTo("28721c6f-b78f-4d5c-a048-19fd2fc429d2");
	}
}