/*
 * Copyright (C) 2023 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.notification.postfach;

import org.springframework.stereotype.Service;

import de.ozgcloud.nachrichten.postfach.GrpcIsPostfachConfiguredRequest;
import de.ozgcloud.nachrichten.postfach.PostfachServiceGrpc.PostfachServiceBlockingStub;
import de.ozgcloud.notification.NotificationManagerCallContextAttachingInterceptor;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Service
class PostfachRemoteService {

	@GrpcClient("pluto")
	private PostfachServiceBlockingStub postfachServiceBlockingStub;

	public boolean isPostfachConfigured() {
		return getPostfachServiceStub().isPostfachConfigured(buildGrpcRequest()).getIsConfigured();
	}

	GrpcIsPostfachConfiguredRequest buildGrpcRequest() {
		return GrpcIsPostfachConfiguredRequest.newBuilder().build();
	}

	PostfachServiceBlockingStub getPostfachServiceStub() {
		return postfachServiceBlockingStub.withInterceptors(new NotificationManagerCallContextAttachingInterceptor());
	}
}
