/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.notification.vorgang;

import java.util.Optional;

import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.NullValueCheckStrategy;
import org.mapstruct.NullValuePropertyMappingStrategy;

import de.ozgcloud.notification.postfach.PostfachAddress;
import de.ozgcloud.notification.postfach.PostfachAddressIdentifier;
import de.ozgcloud.notification.postfach.StringBasedIdentifier;
import de.ozgcloud.vorgang.common.GrpcObject;
import de.ozgcloud.vorgang.vorgang.GrpcPostfachAddress;
import de.ozgcloud.vorgang.vorgang.GrpcServiceKonto;
import de.ozgcloud.vorgang.vorgang.GrpcVorgangHead;
import de.ozgcloud.vorgang.vorgang.GrpcVorgangWithEingang;

@Mapper(nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.SET_TO_NULL,
		nullValueCheckStrategy = NullValueCheckStrategy.ALWAYS)
interface VorgangMapper {

	@Mapping(target = "postfachId", source = "eingang.antragsteller.postfachId")
	@Mapping(target = "vorgangName", source = "name")
	@Mapping(target = "vorgangNummer", source = "nummer")
	@Mapping(target = "organisationseinheitenId", source = "eingang.zustaendigeStelle.organisationseinheitenId")
	@Mapping(target = "postfachAddress", expression = "java(fromPostfachAddress(vorgang))")
	Vorgang fromGrpc(GrpcVorgangWithEingang vorgang);
	
	default PostfachAddress fromPostfachAddress(GrpcVorgangWithEingang vorgang) {
		return Optional.ofNullable(vorgang.getHeader())
			.map(GrpcVorgangHead::getServiceKonto)
			.map(GrpcServiceKonto::getPostfachAddressesList)
			.filter(list -> !list.isEmpty())
			.map(list -> list.get(0))
			.map(this::buildPostfachAddress)
			.orElse(null);
	}
	
	private PostfachAddress buildPostfachAddress(GrpcPostfachAddress postfachAddress) {
		return PostfachAddress.builder()
				.type(postfachAddress.getType())
				.version(postfachAddress.getVersion())
				.identifier(buildIdentifier(postfachAddress.getIdentifier()))
				.build();
	}
	
	private PostfachAddressIdentifier buildIdentifier(GrpcObject identifier) {
		var postfachId = identifier.getProperty(0).getValue(0);
		return StringBasedIdentifier.builder().postfachId(postfachId).build();
	}

	default VorgangId fromString(String vorgangId) {
		return Optional.ofNullable(vorgangId).map(VorgangId::from).orElse(null);
	}
}
