/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.notification.command;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Spy;

import de.ozgcloud.notification.postfach.PostfachNachrichtTestFactory;
import de.ozgcloud.notification.vorgang.VorgangId;
import de.ozgcloud.vorgang.common.grpc.GrpcObjectMapper;

class CommandMapperTest {

	@InjectMocks
	private CommandMapper mapper = Mappers.getMapper(CommandMapper.class);
	@Spy
	private GrpcObjectMapper objectMapper = Mappers.getMapper(GrpcObjectMapper.class);

	@DisplayName("To grpc")
	@Nested
	class TestToGrpc {
		
		@Test
		void shouldMapPostfachNachrichtCommand() {
			var mapped = mapper.toGrpcRequest(CommandTestFactory.createBuilder().body(PostfachNachrichtTestFactory.createAsMap()).build());

			assertThat(mapped).usingRecursiveComparison()
					.ignoringFields("bodyObj_", "memoizedHashCode")
					.isEqualTo(GrpcCreateCommandRequestTestFactory.create());
		}
		
		@DisplayName("to string")
		@Nested
		class TestToString {
			
			@Test
			void shouldReturnValue() {
				var vorgangId = mapper.toString(VorgangId.from("testId"));
				
				assertThat(vorgangId).isEqualTo("testId");
			}
			
			@Test
			void shouldReturnNull() {
				var vorgangId = mapper.toString(null);
				
				assertThat(vorgangId).isNull();				
			}
		}
	}
}