/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.notification.email;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.nachrichten.email.EmailServiceGrpc.EmailServiceBlockingStub;
import de.ozgcloud.nachrichten.email.GrpcRecipient;
import de.ozgcloud.nachrichten.email.GrpcSendEmailRequest;
import de.ozgcloud.notification.NotificationManagerCallContextAttachingInterceptor;

class EmailRemoteServiceTest {

	@InjectMocks
	private EmailRemoteService remoteService;

	@Mock
	private EmailServiceBlockingStub emailStub;

	@Mock
	private EmailRecipientMapper recipientMapper;

	@Nested
	class TestSendNotification {

		@Captor
		private ArgumentCaptor<GrpcSendEmailRequest> requestCaptor;

		private final static GrpcRecipient GRPC_RECIPIENT = GrpcEmailRecipientTestFactory.create();

		@BeforeEach
		void initTest() {
			when(recipientMapper.toGrpc(any())).thenReturn(GRPC_RECIPIENT);
			when(emailStub.withInterceptors(any())).thenReturn(emailStub);
		}

		@Test
		void shouldCallEmailServiceOnlyOnce() {
			remoteService.sendEmail(UserEmailTestFactory.create());

			verify(emailStub, times(1)).sendEmail(any());
		}

		@Test
		void shouldNotifyAllRecipients() {
			remoteService.sendEmail(UserEmailTestFactory.create());

			verify(emailStub).sendEmail(requestCaptor.capture());
			assertThat(requestCaptor.getValue().getRecipientsList()).hasSameSizeAs(UserEmailTestFactory.RECIPIENTS);
		}

		@Test
		void shouldConvertCorrectly() {
			remoteService.sendEmail(UserEmailTestFactory.create());

			verify(emailStub).sendEmail(requestCaptor.capture());
			assertThat(requestCaptor.getValue().getRecipientsList()).first().usingRecursiveComparison().isEqualTo(GRPC_RECIPIENT);
		}

		@Test
		void shouldUseInterceptor() {
			remoteService.sendEmail(UserEmailTestFactory.create());

			verify(emailStub).withInterceptors(any(NotificationManagerCallContextAttachingInterceptor.class));
		}

	}
}