/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.notification.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.notification.NotificationManagerCallContextAttachingInterceptor;
import de.ozgcloud.vorgang.vorgang.GrpcFindVorgangWithEingangRequest;
import de.ozgcloud.vorgang.vorgang.GrpcFindVorgangWithEingangResponse;
import de.ozgcloud.vorgang.vorgang.GrpcVorgangWithEingang;
import de.ozgcloud.vorgang.vorgang.VorgangServiceGrpc.VorgangServiceBlockingStub;

class VorgangRemoteServiceTest {

	@InjectMocks
	private VorgangRemoteService remoteService;
	@Mock
	private VorgangServiceBlockingStub stub;
	@Mock
	private VorgangMapper mapper;

	@Nested
	class TestFindVorgangById {

		private static final GrpcVorgangWithEingang GRPC_VORGANG = GrpcVorgangTestFactory.create();
		private static final Vorgang VORGANG = VorgangTestFactory.create();

		@Captor
		private ArgumentCaptor<GrpcFindVorgangWithEingangRequest> requestCaptor;

		@BeforeEach
		void initStubMock() {
			var response = GrpcFindVorgangWithEingangResponse.newBuilder().setVorgangWithEingang(GRPC_VORGANG).build();
			when(stub.findVorgangWithEingang(any())).thenReturn(response);

			when(stub.withInterceptors(any())).thenReturn(stub);
		}

		@BeforeEach
		void initMapperMock() {
			when(mapper.fromGrpc(any())).thenReturn(VORGANG);
		}

		@Test
		void shouldUseInterceptor() {
			remoteService.getVorgang(VorgangTestFactory.ID);

			verify(stub).withInterceptors(any(NotificationManagerCallContextAttachingInterceptor.class));
		}

		@Test
		void shouldCallStub() {
			remoteService.getVorgang(VorgangTestFactory.ID);

			verify(stub).findVorgangWithEingang(requestCaptor.capture());

			assertThat(requestCaptor.getValue().getId()).isEqualTo(VorgangTestFactory.ID.toString());
		}

		@Test
		void shouldCallMapper() {
			remoteService.getVorgang(VorgangTestFactory.ID);

			verify(mapper).fromGrpc(same(GRPC_VORGANG));
		}

		@Test
		void shouldReturnMappedVorgang() {
			var result = remoteService.getVorgang(VorgangTestFactory.ID);

			assertThat(result).isSameAs(VORGANG);
		}

	}

}
