/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.notification.vorgang;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

class VorgangServiceTest {

	@InjectMocks
	private VorgangService service;

	@Mock
	private VorgangRemoteService remoteService;

	@Nested
	class TestGetVorgang {
		@Test
		void shouldCallRemoteService() {
			service.getVorgang(VorgangTestFactory.ID);

			verify(remoteService).getVorgang(VorgangTestFactory.ID);
		}

		@Test
		void shouldReturnResultFromRemoteService() {
			var vorgang = VorgangTestFactory.create();
			when(remoteService.getVorgang(any())).thenReturn(vorgang);

			var result = service.getVorgang(VorgangTestFactory.ID);

			assertThat(result).isSameAs(vorgang);
		}
	}

}
