/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.attached_item;

import java.util.Optional;
import java.util.stream.Stream;

import org.springframework.beans.factory.annotation.Autowired;

import de.ozgcloud.vorgang.common.security.PolicyService;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcFindVorgangAttachedItemResponse;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItemRequest;
import de.ozgcloud.vorgang.vorgangAttachedItem.GrpcVorgangAttachedItemResponse;
import de.ozgcloud.vorgang.vorgangAttachedItem.VorgangAttachedItemServiceGrpc.VorgangAttachedItemServiceImplBase;
import io.grpc.stub.StreamObserver;
import net.devh.boot.grpc.server.service.GrpcService;

@GrpcService
public class GrpcVorgangAttachedItemService extends VorgangAttachedItemServiceImplBase {

	@Autowired
	private VorgangAttachedItemService service;
	@Autowired
	private GrpcVorgangAttachedItemMapper mapper;
	@Autowired
	private PolicyService policyService;

	@Override
	public void find(GrpcFindVorgangAttachedItemRequest request, StreamObserver<GrpcFindVorgangAttachedItemResponse> responseObserver) {
		policyService.checkPermission(request.getVorgangId());

		var vorgangAttachedItems = service.find(request.getVorgangId(), Optional.of(request.getClient()), Optional.of(request.getItemName()));

		responseObserver.onNext(buildFindResponse(vorgangAttachedItems));
		responseObserver.onCompleted();
	}

	private GrpcFindVorgangAttachedItemResponse buildFindResponse(Stream<VorgangAttachedItem> vorgangAttachedItems) {
		var items = vorgangAttachedItems.map(mapper::mapTo).toList();
		return GrpcFindVorgangAttachedItemResponse.newBuilder().addAllVorgangAttachedItems(items).build();
	}

	@Override
	public void getById(GrpcVorgangAttachedItemRequest request, StreamObserver<GrpcVorgangAttachedItemResponse> responseObserver) {
		policyService.checkPermissionByVorgangAttachedItem(request.getId());

		var vorgangAttachedItem = mapper.mapTo(service.getById(request.getId()));

		responseObserver.onNext(GrpcVorgangAttachedItemResponse.newBuilder().setVorgangAttachedItem(vorgangAttachedItem).build());
		responseObserver.onCompleted();
	}
}