package de.ozgcloud.vorgang.registry;

import java.util.List;
import java.util.Optional;

import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import lombok.extern.log4j.Log4j2;

@Service
@Log4j2
class RegistryService {
	@Value(value = "${ozgcloud.vorgang-manager.address:#{null}}")
	private Optional<String> vorgangManagerAddress;

	@Autowired
	private ZufiRemoteService zufiRemoteService;

	public void registerVorgangManager(List<String> supportedOrganisationsEinheiten) {
		try {
			register(supportedOrganisationsEinheiten);
		} catch (Exception e) {
			LOG.error("Error registering this VorgangsManager.", e);
		}
	}

	private void register(List<String> supportedOrganisationsEinheiten) {
		if (CollectionUtils.isNotEmpty(supportedOrganisationsEinheiten)) {
			registerAddress(supportedOrganisationsEinheiten, vorgangManagerAddress);
		} else {
			LOG.warn("No OrganistationsEinheitenIds provided. Not registering this VorgangsManager");
		}
	}

	void registerAddress(List<String> supportedOrganisationsEinheiten, Optional<String> vorgangManagerAddress) {
		vorgangManagerAddress.ifPresentOrElse(address -> {
			zufiRemoteService.registerVorgangManager(supportedOrganisationsEinheiten, address);
		}, () -> LOG.error("Property ozgcloud.vorgang-manager.address not set. Not registering this VorgangsManager"));
	}
}
