package de.ozgcloud.vorgang.registry;

import java.util.List;

import org.springframework.stereotype.Service;

import de.ozgcloud.vorgang.callcontext.VorgangManagerClientCallContextAttachingInterceptor;
import de.ozgcloud.zufi.grpc.registration.GrpcVorgangManagerRegistrationRequest;
import de.ozgcloud.zufi.grpc.registration.VorgangManagerRegistrationServiceGrpc.VorgangManagerRegistrationServiceBlockingStub;
import lombok.NonNull;
import lombok.extern.log4j.Log4j2;
import net.devh.boot.grpc.client.inject.GrpcClient;

@Log4j2
@Service
class ZufiRemoteService {
	@GrpcClient("zufi-manager")
	private VorgangManagerRegistrationServiceBlockingStub serviceStub;

	public void registerVorgangManager(@NonNull List<String> organistationsEinheitenIds, String vorgangManagerAddress) {
		boolean success = getServiceStub()
				.register(buildRequest(organistationsEinheitenIds, vorgangManagerAddress)).getSuccess();
		LOG.info("Register success: " + success);
	}

	private VorgangManagerRegistrationServiceBlockingStub getServiceStub() {
		return serviceStub.withInterceptors(new VorgangManagerClientCallContextAttachingInterceptor());
	}

	private GrpcVorgangManagerRegistrationRequest buildRequest(List<String> organistationsEinheitenIds, String address) {
		return GrpcVorgangManagerRegistrationRequest.newBuilder().addAllOrganisationsEinheitenIds(organistationsEinheitenIds)
				.setServiceAddress(address).build();
	}
}
