/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.vorgang.command;

import static org.assertj.core.api.Assertions.assertThat;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;

import de.ozgcloud.vorgang.grpc.command.GrpcCommandResponse;
import de.ozgcloud.vorgang.grpc.command.GrpcCommandResponse.GrpcResponseCode;
import de.ozgcloud.vorgang.grpc.command.GrpcOrder;

class CommandResponseMapperTest {

	private GrpcCommandResponseMapper mapper = Mappers.getMapper(GrpcCommandResponseMapper.class);

	@Nested
	class TestCommandResponseToGrpcCommandResponse {

		@Test
		void shouldMapResponseCode() {

			CommandResponse response = CommandResponseTestFactory.create();

			GrpcCommandResponse grpcResponse = mapper.toGrpc(response);

			assertThat(grpcResponse.getResponseCode()).isEqualTo(GrpcResponseCode.PENDING);
		}

		@Test
		void shouldMapMessageCode() {

			CommandResponse response = CommandResponseTestFactory.create();

			GrpcCommandResponse grpcResponse = mapper.toGrpc(response);

			assertThat(grpcResponse.getMessageCode()).isEqualTo(CommandResponseTestFactory.MESSAGE_CODE);
		}

		@Test
		void shouldMapMessageParametersInCorrectOrder() {

			CommandResponse response = CommandResponseTestFactory.create();

			GrpcCommandResponse grpcResponse = mapper.toGrpc(response);

			assertThat(grpcResponse.getMessageParametersList()).isNotEmpty();
			assertThat(grpcResponse.getMessageParametersList()).containsExactly(
					CommandResponseTestFactory.MESSAGE_PARAM1,
					CommandResponseTestFactory.MESSAGE_PARAM2);
		}

		@Test
		void shouldHaveVorgangCommand() {

			CommandResponse response = CommandResponseTestFactory.create();

			GrpcCommandResponse grpcResponse = mapper.toGrpc(response);

			assertThat(grpcResponse.getCommand()).isNotNull();
		}

		@Test
		void shouldMapCommandId() {

			CommandResponse response = CommandResponseTestFactory.create();

			GrpcCommandResponse grpcResponse = mapper.toGrpc(response);

			assertThat(grpcResponse.getCommand().getId()).isEqualTo(CommandTestFactory.ID);
		}

		@Test
		void shouldMapCommandCreatedAt() {

			CommandResponse response = CommandResponseTestFactory.create();

			GrpcCommandResponse grpcResponse = mapper.toGrpc(response);

			assertThat(grpcResponse.getCommand().getCreatedAt()).isEqualTo(CommandTestFactory.CREATED_AT_STR);
		}

		@Test
		void shouldMapCommandCreatedBy() {

			CommandResponse response = CommandResponseTestFactory.create();

			GrpcCommandResponse grpcResponse = mapper.toGrpc(response);

			assertThat(grpcResponse.getCommand().getCreatedBy()).isEqualTo(CommandTestFactory.CREATED_BY);
		}

		@Test
		void shouldMapCommandStatus() {

			CommandResponse response = CommandResponseTestFactory.create();

			GrpcCommandResponse grpcResponse = mapper.toGrpc(response);

			assertThat(grpcResponse.getCommand().getStatus()).isEqualTo(CommandTestFactory.STATUS.name());
		}

		@Test
		void shouldMapCommandVorgangId() {

			CommandResponse response = CommandResponseTestFactory.create();

			GrpcCommandResponse grpcResponse = mapper.toGrpc(response);

			assertThat(grpcResponse.getCommand().getRelationId()).isEqualTo(CommandTestFactory.RELATION_ID);
		}

		@Test
		void shouldMapCommandOrder() {

			CommandResponse response = CommandResponseTestFactory.create();

			GrpcCommandResponse grpcResponse = mapper.toGrpc(response);

			assertThat(grpcResponse.getCommand().getOrder()).isEqualTo(GrpcOrder.VORGANG_ANNEHMEN);
		}
	}
}
